@@ -1,12 +1,20 @@
+#########
+# Author:        aj
+# Maintainer:    $Author: andyjenkinson $
+# Created:       2006
+# Last Modified: $Date: 2009-02-18 13:47:33 +0000 (Wed, 18 Feb 2009) $
+# Id:            $Id: ensembl.pm 569 2009-02-18 13:47:33Z andyjenkinson $
+# Source:        $Source$
+# $HeadURL: https://proserver.svn.sourceforge.net/svnroot/proserver/tags/spec-1.53/lib/Bio/Das/ProServer/SourceAdaptor/Transport/ensembl.pm $
+#
 package Bio::Das::ProServer::SourceAdaptor::Transport::ensembl;
-
 use strict;
+use warnings;
 use Carp;
 use Bio::EnsEMBL::Registry;
-use Bio::EnsEMBL::DBSQL::DBAdaptor;
 use base qw(Bio::Das::ProServer::SourceAdaptor::Transport::generic);
 
-our $VERSION  = do { my @r = (q$Revision: 2.70 $ =~ /\d+/mxg); sprintf '%d.'.'%03d' x $#r, @r };
+our $VERSION  = do { my ($v) = (q$Revision: 569 $ =~ /\d+/mxg); $v; };
 
 sub init {
   my ($self) = @_;
@@ -14,39 +22,78 @@ sub init {
   $self->{'_group'}   = $self->config->{'group'};
   $self->_apply_override;
   $self->_load_registry;
+  return;
 }
 
 sub _load_registry {
   my ($self) = @_;
-  Bio::EnsEMBL::Registry->load_registry_from_db(
-    -host => 'ensembldb.ensembl.org',
-    -user => 'anonymous',
-    -verbose => $self->{'debug'} );
+  if (!$self->config->{'skip_registry'}) {
+    Bio::EnsEMBL::Registry->load_registry_from_db(
+      -host => 'ensembldb.ensembl.org',
+      -user => 'anonymous',
+      -verbose => $self->{'debug'} );
+  }
   Bio::EnsEMBL::Registry->set_disconnect_when_inactive();
+  return;
 }
 
 sub _apply_override {
   my ($self) = @_;
   my $dbname = $self->config->{'dbname'};
+
   if ($dbname) {
-    $self->{'debug'} && carp "Overriding database with $dbname\n";
-    my ($species, $group) = $dbname =~ m/([a-z_]+)_([a-z]+)_\d+/;
-    $species || croak "Unknown database to override: $dbname";
-    $species = 'multi' if ($species  eq 'ensembl');
-    $self->{'_species'} = $species;
-    $self->{'_group'}   = $group;
-    
+
+    my ($species, $group) = $dbname =~ m/([a-z_]+)_([a-z]+)_\d+/mx;
+    if ($species  eq 'ensembl') {
+      $species = 'multi';
+    }
+    $self->{'_species'} ||= $species;
+    $self->{'_group'}   ||= $group;
+
+    if (!$self->{'_species'} || !$self->{'_group'}) {
+      croak "Unable to parse database species and group: $dbname";
+    }
+
+    $self->{'debug'} && carp sprintf "Overriding database with %s (%s,%s)\n",
+                                     $dbname, $self->{'_species'}, $self->{'_group'};
+
+    # This is a map from group names to Ensembl DB adaptors.
+    # Taken from Bio::EnsEMBL::Registry
+    my %group2adaptor = (
+      'blast'   => 'Bio::EnsEMBL::External::BlastAdaptor',
+      'compara' => 'Bio::EnsEMBL::Compara::DBSQL::DBAdaptor',
+      'core'    => 'Bio::EnsEMBL::DBSQL::DBAdaptor',
+      'estgene' => 'Bio::EnsEMBL::DBSQL::DBAdaptor',
+      'funcgen' => 'Bio::EnsEMBL::Funcgen::DBSQL::DBAdaptor',
+      'haplotype' =>
+        'Bio::EnsEMBL::ExternalData::Haplotype::DBAdaptor',
+      'hive' => 'Bio::EnsEMBL::Hive::DBSQL::DBAdaptor',
+      'lite' => 'Bio::EnsEMBL::Lite::DBAdaptor',
+      'otherfeatures' => 'Bio::EnsEMBL::DBSQL::DBAdaptor',
+      'pipeline' =>
+        'Bio::EnsEMBL::Pipeline::DBSQL::DBAdaptor',
+      'snp' =>
+        'Bio::EnsEMBL::ExternalData::SNPSQL::DBAdaptor',
+      'variation' =>
+        'Bio::EnsEMBL::Variation::DBSQL::DBAdaptor',
+      'vega' => 'Bio::EnsEMBL::DBSQL::DBAdaptor',
+    );
+
+    my $adaptorclass = $group2adaptor{ $self->{'_group'} } || croak 'Unknown database group: '.$self->{'_group'};
     # Creating a new connection will add it to the registry.
-    my $dba = Bio::EnsEMBL::DBSQL::DBAdaptor->new(
-      -host    => $self->config->{'host'}     || "localhost",
-      -port    => $self->config->{'port'}     || "3306",
-      -user    => $self->config->{'username'} || "ensro",
+    eval "require $adaptorclass";
+    $@ && die $@;
+    $adaptorclass->new(
+      -host    => $self->config->{'host'}     || 'localhost',
+      -port    => $self->config->{'port'}     || '3306',
+      -user    => $self->config->{'username'} || 'ensro',
       -pass    => $self->config->{'password'},
       -dbname  => $dbname,
-      -species => $species,
-      -group   => $group,
+      -species => $self->{'_species'},
+      -group   => $self->{'_group'},
     );
   }
+  return;
 }
 
 sub adaptor {
@@ -68,7 +115,7 @@ sub slice_adaptor {
   return $self->adaptor($species, $group)->get_SliceAdaptor();
 }
 
-sub chromosome_by_region {
+sub chromosome_by_region { ## no critic
   my ($self, $chr, $start, $end, $species, $group) = @_;
   return $self->slice_adaptor($species, $group)->fetch_by_region('chromosome', $chr, $start, $end);
 }
@@ -96,22 +143,22 @@ sub version {
 sub last_modified {
   my ($self) = @_;
   my $dbc = $self->adaptor()->dbc();
-  my $sth = $dbc->prepare("SHOW TABLE STATUS");
-  $sth->execute;
-  my $server_text = [sort { $b cmp $a }
+  my $sth = $dbc->prepare(q(SHOW TABLE STATUS));
+  $sth->execute();
+  my $server_text = [sort { $b cmp $a } ## no critic
                      keys %{ $sth->fetchall_hashref('Update_time') }
                     ]->[0]; # server local time
-  $sth->finish;
-  $sth = $dbc->prepare("select UNIX_TIMESTAMP(?) as 'unix'");
+  $sth->finish();
+  $sth = $dbc->prepare(q(SELECT UNIX_TIMESTAMP(?) as 'unix'));
   $sth->execute($server_text); # sec since epoch
   my $server_unix = $sth->fetchrow_arrayref()->[0];
-  $sth->finish;
-  return $server_unix;  
+  $sth->finish();
+  return $server_unix;
 }
 
 sub disconnect {
   my $self = shift;
-  Bio::EnsEMBL::Registry->disconnect_all;
+  Bio::EnsEMBL::Registry->disconnect_all();
   $self->{'debug'} and carp "$self performed disconnect\n";
   return;
 }
@@ -121,11 +168,11 @@ __END__
 
 =head1 NAME
 
-Bio::Das::ProServer::SourceAdaptor::Transport::ensembl_registry
+Bio::Das::ProServer::SourceAdaptor::Transport::ensembl
 
 =head1 VERSION
 
-$Revision: 2.70 $
+$LastChangedRevision: 569 $
 
 =head1 SYNOPSIS
 
@@ -139,20 +186,16 @@ and can be used in a species specific or cross-species manner. The main
 advantage of using this Transport is that the registry automatically provides
 access to the latest data available to the installed API.
 
-=head1 AUTHOR
+=head1 SUBROUTINES/METHODS
 
-Andy Jenkinson <andy.jenkinson@ebi.ac.uk>
-
-=head1 METHODS
-
-=head2 init : Post-construction initialisation.
+=head2 init - Post-construction initialisation.
 
   $oTransport->init();
   
   Loads the registry from the Ensembl database, and applies a custom database
   override if specified.
 
-=head2 adaptor : Gets an Ensembl adaptor.
+=head2 adaptor - Gets an Ensembl adaptor.
 
   $oAdaptor = $oTransport->adaptor();
   $oAdaptor = $oTransport->adaptor('human', 'core');
@@ -163,7 +206,7 @@ Andy Jenkinson <andy.jenkinson@ebi.ac.uk>
   Returns:
     L<Bio::EnsEMBL::DBSQL::DBAdaptor>
 
-=head2 slice_adaptor : Gets an Ensembl slice adaptor.
+=head2 slice_adaptor - Gets an Ensembl slice adaptor.
   
   $oAdaptor = $oTransport->slice_adaptor();
   $oAdaptor = $oTransport->slice_adaptor('human', 'core');
@@ -174,7 +217,7 @@ Andy Jenkinson <andy.jenkinson@ebi.ac.uk>
   Returns:
     L<Bio::EnsEMBL::DBSQL::SliceAdaptor>
 
-=head2 gene_adaptor : Gets an Ensembl gene adaptor.
+=head2 gene_adaptor - Gets an Ensembl gene adaptor.
   
   $oAdaptor = $oTransport->gene_adaptor();
   $oAdaptor = $oTransport->gene_adaptor('human', 'core');
@@ -185,7 +228,7 @@ Andy Jenkinson <andy.jenkinson@ebi.ac.uk>
   Returns:
     L<Bio::EnsEMBL::DBSQL::GeneAdaptor>
 
-=head2 chromosome_by_region : Gets a chromosome slice.
+=head2 chromosome_by_region - Gets a chromosome slice.
 
   $oSlice = $oTransport->chromosome_by_region('X');
   $oSlice = $oTransport->chromosome_by_region('X', 123453, 132424);
@@ -200,7 +243,7 @@ Andy Jenkinson <andy.jenkinson@ebi.ac.uk>
   Returns:
     L<Bio::EnsEMBL::Slice>
 
-=head2 chromosomes : Gets all chromosomes.
+=head2 chromosomes - Gets all chromosomes.
 
   $aSlices = $oTransport->chromosomes();
   $aSlices = $oTransport->chromosomes('human', 'core');
@@ -211,7 +254,7 @@ Andy Jenkinson <andy.jenkinson@ebi.ac.uk>
   Returns:
     listref of L<Bio::EnsEMBL::Slice> objects
 
-=head2 gene_by_id : Gets a gene.
+=head2 gene_by_id - Gets a gene.
 
   $oGene = $oTransport->gene_by_id('ENSG00000139618'); # BRCA2
   $oGene = $oTransport->gene_by_id('ENSG00000139618', 'human', 'core');
@@ -223,7 +266,7 @@ Andy Jenkinson <andy.jenkinson@ebi.ac.uk>
   Returns:
     L<Bio::EnsEMBL::Gene>
 
-=head2 genes : Gets all genes.
+=head2 genes - Gets all genes.
 
   $aGenes = $oTransport->genes();
   $aGenes = $oTransport->genes('human', 'core');
@@ -234,15 +277,15 @@ Andy Jenkinson <andy.jenkinson@ebi.ac.uk>
   Returns:
     listref of L<Bio::EnsEMBL::Gene> objects
 
-=head2 version : Gets the Ensembl API's release number.
+=head2 version - Gets the Ensembl API's release number.
 
   $sVersion = $oTransport->version();
   
-=head2 last_modified : Gets a last modified date from the database.
+=head2 last_modified - Gets a last modified date from the database.
 
   $sVersion = $oTransport->version();
 
-=head2 disconnect : ProServer hook to disconnect all connected databases.
+=head2 disconnect - ProServer hook to disconnect all connected databases.
 
   $oTransport->disconnect();
 
@@ -250,17 +293,26 @@ Andy Jenkinson <andy.jenkinson@ebi.ac.uk>
 
 Configured as part of each source's ProServer 2 INI file.
 
-  The 'default database' is configured using these properties:
-    species  (defaults to human)
-    group    (defaults to core)
-
-  A specific database may be overridden using these properties:
+  The transport will automatically load database connection settings from
+  the Ensembl Registry at ensembldb.ensembl.org. To skip this, set the
+  'skip_registry' INI property.
+  
+  A specific database may also be overridden using these INI properties:
     dbname
     host     (defaults to localhost)
     port     (defaults to 3306)
     username (defaults to ensro)
     password
 
+  The 'default database' used in the transport's data access methods may be
+  configured using these INI properties:
+    species  (defaults to human)
+    group    (defaults to core)
+
+=head1 DIAGNOSTICS
+
+=head1 CONFIGURATION AND ENVIRONMENT
+
 =head1 DEPENDENCIES
 
 =over
@@ -269,17 +321,9 @@ Configured as part of each source's ProServer 2 INI file.
 
 =item L<Bio::Das::ProServer::SourceAdaptor::Transport::generic>
 
-=item L<Bio::EnsEMBL::Registry>
+=item Ensembl core API
 
-=item L<Bio::EnsEMBL::DBSQL::DBAdaptor>
-
-=back
-
-=head1 REFERENCES
-
-=over
-
-=item L<http://www.ensembl.org/info/software/Pdoc/ensembl/> Ensembl API
+=item Additional Ensembl APIs if used
 
 =back
 
@@ -291,8 +335,18 @@ None reported
 
 None reported
 
+=head1 REFERENCES
+
+=over
+
+=item L<http://www.ensembl.org/info/software/Pdoc/ensembl/> Ensembl API
+
+=back
+
+=head1 AUTHOR
+
+Andy Jenkinson <andy.jenkinson@ebi.ac.uk>
+
 =head1 LICENSE AND COPYRIGHT
 
 Copyright (c) 2007 EMBL-EBI
-
-=cut
