@@ -2,22 +2,23 @@
 # Author:        rmp
 # Maintainer:    rmp
 # Created:       2003-12-12
-# Last Modified: $Date: 2007/11/20 20:12:21 $ $Author: rmp $
-# Id:            $Id: dbi.pm,v 2.70 2007/11/20 20:12:21 rmp Exp $
-# Source:        $Source: /cvsroot/Bio-Das-ProServer/Bio-Das-ProServer/lib/Bio/Das/ProServer/SourceHydra/dbi.pm,v $
-# $HeadURL$
+# Last Modified: $Date: 2008-12-10 12:30:40 +0000 (Wed, 10 Dec 2008) $ $Author: andyjenkinson $
+# Id:            $Id: dbi.pm 559 2008-12-10 12:30:40Z andyjenkinson $
+# Source:        $Source: /nfs/team117/rmp/tmp/Bio-Das-ProServer/Bio-Das-ProServer/lib/Bio/Das/ProServer/SourceHydra/dbi.pm,v $
+# $HeadURL: https://proserver.svn.sourceforge.net/svnroot/proserver/tags/spec-1.53/lib/Bio/Das/ProServer/SourceHydra/dbi.pm $
 #
 # DBI-driven sourceadaptor broker
 #
 package Bio::Das::ProServer::SourceHydra::dbi;
 use strict;
 use warnings;
-use base qw(Bio::Das::ProServer::SourceHydra);
 use English qw(-no_match_vars);
 use Carp;
+use base qw(Bio::Das::ProServer::SourceHydra);
+use Readonly;
 
-our $VERSION       = do { my @r = (q$Revision: 2.70 $ =~ /\d+/mxg); sprintf '%d.'.'%03d' x $#r, @r };
-our $CACHE_TIMEOUT = 30;
+our $VERSION = do { my ($v) = (q$Revision: 559 $ =~ /\d+/mxg); $v; };
+Readonly::Scalar our $CACHE_TIMEOUT => 30;
 
 #########
 # the purpose of this module:
@@ -51,23 +52,50 @@ sub sources {
       $sth->execute();
 
       $self->{'_tables'} = [map {
-	my ($remainder) = $_->[0] =~ /^.{$l}(.*)$/mx;
-	$dsn.$remainder;
+	$dsn.($_->[0] =~ /^.{$l}(.*)$/mx)[0];
       } @{$sth->fetchall_arrayref()}];
 
       $sth->finish();
       $self->{'debug'} and carp qq(@{[scalar @{$self->{'_tables'}}]} tables found);
-    };
+      1;
 
-    if($EVAL_ERROR) {
+    } or do {
       carp "Error scanning tables: $EVAL_ERROR";
       delete $self->{'_tables'};
-    }
+    };
   }
 
   return @{$self->{'_tables'} || []};
 }
 
+sub last_modified {
+  my $self = shift;
+
+  if($self->transport->dbh->{Driver}->{Name} ne 'mysql') {
+    return;
+  }
+
+  my $now      = time;
+  my $basename = $self->config->{'basename'};
+
+  #########
+  # flush the timestamp cache *at most* once every $CACHE_TIMEOUT
+  # This may need signal triggering to have immediate support
+  #
+  if($now > ($self->{'_lastmodified_timestamp'} || 0)+$CACHE_TIMEOUT) {
+    $self->{'debug'} and carp qq(Flushing last-modified cache for hydra $self->{'dsn'});
+    $self->{'_lastmodified_timestamp'} = $now;
+    my $server_text = [sort { $b cmp $a } ## no critic
+                     map { $_->{Update_time} }
+                     @{ $self->transport()->query(q(SHOW TABLE STATUS),{Update_time=>1}) }
+                    ]->[0]; # server local time
+    my $server_unix = $self->transport()->query(q(SELECT UNIX_TIMESTAMP(?) as 'unix'), $server_text)->[0]{unix}; # sec since epoch
+    $self->{'_lastmodified'} = $server_unix;
+  }
+
+  return $self->{'_lastmodified'};
+}
+
 1;
 __END__
 
@@ -77,7 +105,7 @@ Bio::Das::ProServer::SourceHydra::dbi - A database-backed implementation of B::D
 
 =head1 VERSION
 
-$Revision: 2.70 $
+$Revision: 559 $
 
 =head1 AUTHOR
 
@@ -112,8 +140,17 @@ disclaimers of warranty.
   This routine caches results for $CACHE_TIMEOUT as show tables can be
   slow for a few thousand sources.
 
+=head2 last_modified : machine time of last data change
+
+  Gets the most recent update time for any of the hydra's tables.
+  Only knows how to do this for MySQL databases.
+
+  my $unixtime = $dbihydra->last_modified();
+
 =head1 DIAGNOSTICS
 
+Run ProServer with the -debug flag.
+
 =head1 CONFIGURATION AND ENVIRONMENT
 
   [mysimplehydra]
@@ -134,4 +171,6 @@ Bio::Das::ProServer::SourceHydra
 
 =head1 BUGS AND LIMITATIONS
 
+The last_modified method only works for MySQL databases.
+
 =cut
